//SA bump-shiny material with no reflection. Based on NVIDIA Fx Composer example "BumpGlossed.fx",with tweaks and
//changes as necessary for SA compatability

//////////////////////////////////////////////////
//Matrices
//////////////////////////////////////////////////
half4x4 WorldIT : WorldInverseTranspose < string UIWidget = "None"; >;
half4x4 WorldViewProj : WorldViewProjection < string UIWidget = "None"; >;
half4x4 World : World < string UIWidget = "None"; >;
half4x4 ViewInv : ViewInverse < string UIWidget = "None"; >;

//////////////////////////////////////////////////
//Main Light
//////////////////////////////////////////////////

half3 LightDir = {1.0f, -1.0f, 1.0f};
half4 LightColor1 = {1.0f, 1.0f, 1.0f, 1.0f};

//////////////////////////////////////////////////
//Material surface
//////////////////////////////////////////////////

half4 MatDiffuse = {1.0f, 1.0f, 1.0f, 1.0f};
half4 MatAmbient = {0.0f, 0.0f, 0.0f, 1.0f};
half4 MatSpec = {1.0f, 1.0f, 1.0f, 1.0f};
half MatSpecPow = 40.0;

//////////////////////////////////////////////////
//Samplers
//////////////////////////////////////////////////

sampler2D colorSampler;
sampler2D normalSampler;
samplerCUBE envSampler;

//////////////////////////////////////////////////
//Structures
//////////////////////////////////////////////////

//Vertex structre from game
struct appdata {
    half3 Position	: POSITION;
    half3 UV		: TEXCOORD0;
    half4 Normal	: NORMAL;
    half4 Tangent	: TANGENT0;
    half4 Binormal	: BINORMAL0;
};

//data passed from the vertex shader to the pixel shader
struct vertexOutput {
    half4 HPosition	: POSITION;
    half3 TexCoord	: TEXCOORD0;
    half3 LightVec1	: TEXCOORD1;
    half3 WorldNormal	: TEXCOORD2;
    half3 WorldPos	: TEXCOORD3;
    half3 WorldEyePos	: TEXCOORD4;
    half3 WorldTangent	: TEXCOORD5;
    half3 WorldBinorm	: TEXCOORD6;
};

//////////////////////////////////////////////////
//Shaders
//////////////////////////////////////////////////

//Vertex Shader
vertexOutput mainVS(appdata IN)
{	
	vertexOutput OUT;
    OUT.WorldNormal = mul(IN.Normal, WorldIT).xyz;
    OUT.WorldTangent = mul(IN.Tangent, WorldIT).xyz;
    OUT.WorldBinorm = mul(IN.Binormal, WorldIT).xyz;
    half4 Po = half4(IN.Position.xyz,1.0);
    half3 Pw = mul(Po, World).xyz;
    OUT.WorldPos = Pw;
    OUT.LightVec1 = normalize(-LightDir);
    OUT.TexCoord = IN.UV;
    OUT.WorldEyePos = ViewInv[3].xyz;
    OUT.HPosition = mul(Po, WorldViewProj);
    return OUT;
}

//Pixel Shader
half4 mainPS(vertexOutput IN) : COLOR
{
    half4 map = tex2D(colorSampler,IN.TexCoord.xy);
    half3 bumps = tex2D(normalSampler,IN.TexCoord.xy).xyz - (0.5).xxx;
    half specmap = saturate(tex2D(colorSampler,IN.TexCoord.xy));
    half3 Nn = normalize(IN.WorldNormal);
    half3 Tn = normalize(IN.WorldTangent);
    half3 Bn = normalize(IN.WorldBinorm);
    half3 Nb = Nn + (bumps.x * Tn + bumps.y * Bn);
    Nb = normalize(Nb);
    
    //calc light vec
    half3 Vn = normalize(IN.WorldEyePos - IN.WorldPos);
    half3 Ln = normalize(IN.LightVec1.xyz);
    half3 Hn = normalize(Vn + Ln);
    half hdn = pow(max(0,dot(Hn,Nb)),MatSpecPow) * specmap;
    half ldn = dot(Ln,Nb);
    ldn = max(0.0,ldn);
    
    //blend light colors
    half4 diffContrib = ldn * LightColor1;
    half4 specContrib = (ldn * hdn) * (LightColor1 * MatSpec);
	
	half4 result = (map * MatDiffuse) * MatAmbient + specContrib;     
	result.a = MatDiffuse.a * map.a;
	
    //return result;
	return map;
}